function w = emPCA_L1(X, n_nz)
% Expectation maximization for sparse PCA.
%
% INPUTS
% X: (n x d) zero mean data matrix (samples are rows)
% n_nz: number of nonzero coefficients in w
%
% OUTPUTS
% w: sparse loadings of first principal component (normalized to unit length)
%
% Tuneable parameters are set in the global variable PRM.
%
% Copyright 2013 Christian Sigg (christian@sigg-iten.ch)
% See license file for details.
%
% This code is no longer actively maintained. See my 'nsprcomp' R package
% for a better documented and more feature complete implementation.

global PRM
eps = PRM.emPCA_L1.eps;            % iteration stopping criterion
optimize = PRM.emPCA_L1.optimize;  % optimize weights for given sparsity pattern

[n,d] = size(X);

[xx, w] = PCA(X,1);  % initialize with (full) loadings of first PC
w_old = zeros(size(w));
while(abs(w'*w_old) < 1 - eps)
	w_old = w;
	
	% E step: orthogonally project on w
	y = X*w;

    % M step:
    w = isoQP_L1(y'*y, -(sum(X.*repmat(y,1,d)))', n_nz);
    w = w / norm(w);
end

if optimize
    indx = abs(w) > 0;
    [xx, v] = PCA(X(:,indx),1);
    w = zeros(d,1);
    w(indx) = v;
end
end
