function w = emPCA_L1_NN(X, n_nz)
% Expectation maximization for non-negative sparse PCA. Profits from
% random restarts.
%
% INPUTS
% X: (n x d) zero mean data matrix (samples are rows)
% n_nz: number of nonzero coefficients in w (this is only an upper limit)
%
% OUTPUTS
% w: principal component (normalized to unit length)
%
% Tuneable parameters are set in the global variable PRM.
%
% Copyright 2013 Christian Sigg (christian@sigg-iten.ch)
% See license file for details.
%
% This code is no longer actively maintained. See my 'nsprcomp' R package
% for a better documented and more feature complete implementation.


global PRM
eps = PRM.emPCA_L1_NN.eps;            % iteration stopping criterion
optimize = PRM.emPCA_L1_NN.optimize;  % perform optimization for given sparsity pattern

[n,d] = size(X);

w = iterate(X, n_nz, eps);
if (optimize && n_nz < d)
    indx = w > 0;
    v = iterate(X(:,indx), sum(indx), eps);
    w = zeros(d,1);
    w(indx) = v; 
end
end


function w = iterate(X, n_nz, eps)

[n,d] = size(X);

% initialize with random unit vector in nonnegative orthant
w = abs(randn(d,1));
w = w/norm(w);

w_old = zeros(size(w));
while(abs(w'*w_old) < 1 - eps)
	w_old = w;

    % E step: orthogonally project on w
	y = X*w;

    % M step:
    w = isoQP_L1_NN(y'*y, -(sum(X.*repmat(y,1,d)))', n_nz);
    w = w/norm(w);
end
end
