function w = isoQP_L1(h, f, n_nz)
% Solve min_w h/2*w'*w + f'*w, subject to norm(w,1) <= k, but instead of
% specifying k, the number of nonzero elements n_nz is the criterion.
%
% The QP is of simple form due to the Hessian being a scaled identity
% matrix, and therefore reduces to minimizing quadratic distance to the
% unconstrained minimum.
%
% INPUTS
% h: (non-negative) scalar of the Hessian
% f: linear term
% n_nz: number of nonzero coefficients in w
%
% OUTPUTS
% w: optimum vector satisfying constraints
%
% Copyright 2013 Christian Sigg (christian@sigg-iten.ch)
% See license file for details.
%
% This code is no longer actively maintained. See my 'nsprcomp' R package
% for a better documented and more feature complete implementation.

d = numel(f);
w_star = -f/h;
if (n_nz > d)
    n_nz = d;
end

% do NN monotone stagewise optimization
[ms, mi] = sort(abs(w_star), 'descend');
ms(d+1) = 0;

w = zeros(d,1);
w(1:n_nz) = ms(1:n_nz) - ms(n_nz+1);

% backtransform solution w into correct orthant
[xx,mii] = sort(mi, 'ascend');
w = w(mii);
w = w.*sign(w_star);
end