function w = isoQP_L1_NN(h, f, n_nz)
% Solve min_w h/2*w'*w + w'*f, subject to norm(w,1) <= k and w_i >= 0.
% Instead of specifying k, the number of nonzero elements n_nz is the 
% stopping criterion.
%
% The optimal solution sets w_i = 0 for f_i < 0 and calls isoQP_L1 (without
% the non-negativity constraint) on the remaining variables.
%
% INPUTS
% h: scalar of the Hessian
% f: linear term
% n_nz: number of nonzero coefficients in w (this is only an upper limit)
%
% OUTPUTS
% w: optimum vector satisfying constraints
%
% Copyright 2013 Christian Sigg (christian@sigg-iten.ch)
% See license file for details.
%
% This code is no longer actively maintained. See my 'nsprcomp' R package
% for a better documented and more feature complete implementation.

d = numel(f);
nni = f<0;
if (sum(nni) == 0)
    error('no non-negative components left');
end

w = zeros(d,1);
w(nni) = isoQP_L1(h, f(nni), n_nz);
end